

/******************************************************************************/
/*                                                                            */
/*  receivepacket_ser.c                                                       */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  x = receivepacket_ser( ComPortNr )                                        */
/*                                                                            */
/******************************************************************************/

//mex receivepacket.c winmm.lib libcmt.lib inpout32.lib -D_MT

#include <windows.h>
#include <math.h>

#include "mex.h"


#define PROGNAME "receivepacket_ser"

#define N_BYTES 10	//inclusive start and stop byte
#define STARTBYTE 237 
#define STOPBYTE 77
#define MYBAUDRATE CBR_19200



typedef unsigned char uint8_t;
typedef short int int16_t;
typedef unsigned short int uint16_t;
typedef unsigned int uint32_t;


int receive_packet( HANDLE hComPort, uint8_t* status_packet );


/******************************************************************************/
/*  global variables                                                          */
/******************************************************************************/

HANDLE hComPort = 0;
int ComPortNr_last = 0;




/******************************************************************************/
/*  Exit Function                                                             */
/******************************************************************************/

void MyExitFunction( void )
{
	if (hComPort!=0 )
	{
		CloseHandle(hComPort);
		hComPort = 0;
	}

	ComPortNr_last = 0;
}


/******************************************************************************/
/*  Gateway-Routine                                                           */
/******************************************************************************/

void mexFunction(
                  int nlhs,       mxArray *plhs[],
                  int nrhs, const mxArray *prhs[]
           		 )
{
	mxArray const *parg;
	int m, n, N;

	void *dummy = NULL;
    char PortStr[6], 
		 PortStrFull[_MAX_PATH], 
		 PortTest[6];
	int ComPortNr;
	DCB dcb;
	BOOL bReturn;

	int iPacketIsValid;
	int iTimeout;
	unsigned char status_packet[100];


	// register exit function
	mexAtExit( MyExitFunction );


	/* Check for proper number of arguments */
	if ( (nlhs != 0) && (nlhs != 1) )
		mexErrMsgTxt(PROGNAME " requires 1 output argument.");

	if (nrhs != 1 )
		mexErrMsgTxt(PROGNAME " requires 1 input argument.");

   /* Check dimensions, check input values and assign pointer to the various parameters*/
	// in
	parg = prhs[0];
	m = mxGetM(parg);
	n = mxGetN(parg); 
	N = m*n;
	if ( !mxIsNumeric(parg) || mxIsComplex(parg) || 
		 mxIsSparse(parg)  || !mxIsDouble(parg) || (N!=1) )
		mexErrMsgTxt(PROGNAME " requires that comportnr be a real scalar.");
	else
		ComPortNr = (int)floor( mxGetScalar(prhs[0]) + 0.5 );

	if ( (ComPortNr!=ComPortNr_last) || (hComPort==0) )
	{
		ComPortNr_last = ComPortNr;

		if (hComPort!=0)
		{
			CloseHandle(hComPort);
			hComPort = 0;
		}

		sprintf(PortStr, "com%i", ComPortNr);
		PortStr[5] = '\0';

		/* Test to see if asking for a valid com port */
		strcpy( PortTest, PortStr );
		PortTest[3] = '\0';
		if (strcmp(PortTest,"com")==0)
		{
			if ((PortStr[3]<'1') | (PortStr[3]>'8'))
				mexErrMsgTxt("wrong input argument: comport" );
		}
		else
			mexErrMsgTxt("wrong input argument: comport\n" );

		PortStr[4] = '\0';
		strcpy(PortStrFull, "\\\\.\\");    /* use the full legal device name */
		strcat(PortStrFull, PortStr);

		hComPort = CreateFile( PortStrFull, 
			  GENERIC_READ | GENERIC_WRITE,
			  0,
			  NULL,
			  OPEN_EXISTING,
			  0,
			  NULL );

		if (hComPort == INVALID_HANDLE_VALUE)
		{
			hComPort = 0;
			mexErrMsgTxt("open comport failed" );
		}


		// Omit the call to SetupComm to use the default queue sizes.
		// Get the current configuration.
		bReturn = GetCommState(hComPort, &dcb);
		if (!bReturn) 
		{
			mexErrMsgTxt("GetCommState failed" );
		}

		// Fill in the DCB: baud=9600, 8 data bits, no parity, 1 stop bit. 
		dcb.BaudRate = MYBAUDRATE;
		dcb.ByteSize = 8;
		dcb.Parity = NOPARITY;
		dcb.StopBits = ONESTOPBIT;

		bReturn = SetCommState(hComPort, &dcb);
		if (!bReturn) 
		{
			mexErrMsgTxt("SetCommState failed" );
		}

	}

	iPacketIsValid = receive_packet( hComPort, status_packet );
	iTimeout = 5000;
	while ( (iPacketIsValid<=0) && (iTimeout>0) )
	{
		Sleep(200);
		iTimeout = iTimeout - 200;
		iPacketIsValid = receive_packet( hComPort, status_packet );
	}

	/* Create a matrix for the return arguments */
	if (iPacketIsValid>0)
	{
		int n;
		double *out;

		plhs[0] = mxCreateDoubleMatrix(1, N_BYTES-3, mxREAL);
		out = mxGetPr(plhs[0]);

		for (n=0; n<(N_BYTES-3); n++)
			out[n] = status_packet[n+1];
	}
	else
		plhs[0] = mxCreateDoubleMatrix(1, 0, mxREAL);

}




int receive_packet( HANDLE hComPort, uint8_t* status_packet )
{
	BOOL bResult;
	DWORD nNumberOfBytesToRead;
	DWORD NumberOfBytesRead;

	int iPacketReceived = 0;
	int iPacketIsValid = 0;

	while ( iPacketReceived==0 )
	{
		nNumberOfBytesToRead = 1;
		bResult = ReadFile(hComPort, status_packet, nNumberOfBytesToRead, &NumberOfBytesRead, NULL);
		if ( (!bResult) || (nNumberOfBytesToRead!=NumberOfBytesRead) ) 
			return -1;

		if ( status_packet[0] == STARTBYTE )
		{
			nNumberOfBytesToRead = N_BYTES-1;
			bResult = ReadFile(hComPort, status_packet+1, nNumberOfBytesToRead, &NumberOfBytesRead, NULL);
			if ( (!bResult) || (nNumberOfBytesToRead!=NumberOfBytesRead) )
				return -1;

			iPacketReceived = 1;

			iPacketIsValid=0;
			if (status_packet[N_BYTES-1]==STOPBYTE)
			{
				int n;

				unsigned char sumbyte = 0;
				for ( n=1; n<(N_BYTES-2); n++ )
					sumbyte = (sumbyte + status_packet[n]);
				if (sumbyte==status_packet[N_BYTES-2]) 
					iPacketIsValid = 1;
			}
		}
	}

	return iPacketIsValid;
}


